//+------------------------------------------------------------------+
//|                                                RoofingFilter.mq4 |
//|                                         Copyright  2014, Kirill |
//|                                          StockProgrammer@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2014, Kirill"
#property link      "StockProgrammer@mail.ru"

//+------------------------------------------------------------------+
//| Description                                                      
//|
//| Roofing filter = 2-Pole HighPass Filter + SuperSmoother
//| Recommended for universal use to remove Spectral Dilation
//| 2-Pole - therefore zero mean
//| NOTE: 1-Pole is insufficient to remove Spectral Dilation
//| because market amplitude rises 6 db/octave (Brownian motion ~1/F^2)
//| and a 1-pole filter attenuates at 6 db/octave
//| however, a 2-pole is sufficient, because it attenuates at 12 db/octave
//+------------------------------------------------------------------+


#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 White
//---- indicator parameters
extern int LPPeriod = 40;             
extern int HPPeriod = 80;             
extern int Shift  = 0;                          //Shift Left
//---- indicator buffers
double ExtMapBuffer[];
double HP[];           //used for calculations only
//----
int ExtCountedBars=0;
int BarsUsed = 3;                               //This indicator always needs data from the current bar + only 2 previous bars
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init(){
   int    draw_begin;
   string short_name;
//---- drawing settings
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,3);
   SetIndexShift(0,-1*Shift);   
   
   SetIndexStyle(1,DRAW_NONE);
         
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS));
   
   SetIndexDrawBegin(0,BarsUsed);                                      
//---- indicator short name
   IndicatorShortName("RoofingFilter("+LPPeriod+","+HPPeriod+")");
//---- indicator buffers mapping
   SetIndexBuffer(0,ExtMapBuffer);
   SetIndexBuffer(1,HP);    
//---- initialization done
   return(0);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start(){
   if(Bars<=BarsUsed) return(0);
   ExtCountedBars=IndicatorCounted();
//---- check for possible errors
   if (ExtCountedBars<0) return(-1);
//---- last counted bar will be recounted
   if (ExtCountedBars>0) ExtCountedBars--;
//----
   RoofingFilter();
//---- done
   return(0);
}
//+------------------------------------------------------------------+
//| RoofingFilter Calculation                                        |
//+------------------------------------------------------------------+
void RoofingFilter(){
   double alpha,a,b,c1,c2,c3;
   double pi = 3.1415926535;
   int    i,pos1 = Bars-ExtCountedBars-1;
   int    pos2 = pos1;                       //because we have two cycles in this idicator
//---- calculate constants
   alpha = (MathCos(0.707*2*pi/HPPeriod) + MathSin(0.707*2*pi/HPPeriod) - 1) / MathCos(0.707*2*pi/HPPeriod);
   
   a = MathExp(-1.414 * pi / LPPeriod);
   b = 2 * a * MathCos(1.414 * pi / LPPeriod);
   c2 = b;
   c3 = -1 * a * a;
   c1 = 1 - c2 - c3;

//---- zero initial bars
   if(ExtCountedBars<1)
      for(i=1;i<BarsUsed;i++){
         HP[Bars-i]=0;
         ExtMapBuffer[Bars-i]=0; 
         pos1--;
         pos2--;
      }
//---- calculate all other bars      
   //Highpass filter low-frequency cyclic components
   //Using two-pole Highpass filter to achieve 12db/octave attenuation roll-on (with falling frequency)
   while(pos1>=0)
   {
      HP[pos1] = MathPow((1-alpha/2),2) * (Close[pos1]-2*Close[pos1+1]+Close[pos1+2]) + 2*(1-alpha)*HP[pos1+1] - MathPow((1-alpha),2)*HP[pos1+2];
      
      pos1--;
   }
   //Lowpass filter high-frequency cyclic components
   while(pos2>=0)
   {
      ExtMapBuffer[pos2] = 
                           c1 * (HP[pos2] + HP[pos2+1]) / 2 
                              + c2 * ExtMapBuffer[pos2+1] + c3 * ExtMapBuffer[pos2+2];
      
      pos2--;
   }
}

